package com.tencent.edu.commonview.widget

import android.content.Context
import android.util.AttributeSet
import android.view.ViewGroup
import android.widget.LinearLayout
import androidx.appcompat.widget.AppCompatImageButton
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.module.knowledge.KnowledgeCollectRequester

/**
 * 收藏按钮
 * @author aidanyang
 * @date 2020/7/14
 */
class CollectButton @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : AppCompatImageButton(context, attrs, defStyleAttr) {

    private var requester = KnowledgeCollectRequester()

    private var isCollectAlready = false

    private var contentId: String? = null
    private var serialId: String? = null
    private var externalFlag = 0

    private fun initView(context: Context) {
        scaleType = ScaleType.CENTER_INSIDE
        setImageDrawable(context.resources.getDrawable(R.drawable.icon_collect_notyet))
        val ps = LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT) //(Utils.dp2px(32), Utils.dp2px(32));
        ps.setMargins(0, 0, 0, 0)
        setBackgroundColor(context.resources.getColor(R.color.transparent))
        isClickable = true
        layoutParams = ps
    }

    init {
        initView(context)
    }

    fun initStatus(contentId: String?, externalFlag: Int, serialId: String?) {
        this.contentId = contentId
        this.externalFlag = externalFlag
        this.serialId = serialId

        if (contentId.isNullOrBlank() && serialId.isNullOrBlank()) {
            return
        }

        requester.getCollectStatus(contentId, serialId, externalFlag, object : KnowledgeCollectRequester.GetCollectStatusListener {
            override fun isCollect(isCollect: Boolean) {
                if (isCollect) {
                    isCollectAlready = isCollect
                    ThreadMgr.postToUIThread {
                        setImageResource(R.drawable.icon_collect_already)
                    }
                }
            }
        })
    }

    fun setStatus() {
        if (contentId.isNullOrBlank() && serialId.isNullOrBlank()) {
            return
        }

        requester.updateCollectStatus(contentId, externalFlag, if (isCollectAlready) 1 else 0, serialId, object : KnowledgeCollectRequester.UpdateCollectStatusListenerImpl() {
            override fun onSuccess(type: Int) {
                super.onSuccess(type)
                isCollectAlready = !isCollectAlready
                ThreadMgr.postToUIThread {
                    setImageResource(if (isCollectAlready)
                        R.drawable.icon_collect_already
                    else
                        R.drawable.icon_collect_notyet)
                }
            }
        })
    }
}