/*
 * Copyright © 1998 - 2020 Tencent. All Rights Reserved.
 * 腾讯公司 版权所有
 */
package com.tencent.edu.module.course.task.courseware

import android.content.Context
import android.graphics.Bitmap
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import com.nostra13.universalimageloader.core.ImageLoader
import com.nostra13.universalimageloader.core.assist.FailReason
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener
import com.tencent.edu.R
import com.tencent.edu.common.utils.FileUtils
import com.tencent.edu.module.vodplayer.widget.GifImageViewExt
import uk.co.senab.photoview.PhotoViewAttacher
import java.io.File

/**
 * @author aidanyang
 * @date 2020-6-28
 */
class ImagePreviewView constructor(context: Context) : FrameLayout(context), LifecycleObserver {

    private var giv: GifImageViewExt? = null

    init {
        initView()
    }

    private fun initView() {
        View.inflate(context, R.layout.layout_preview_image, this)
    }

    fun setFile(path: String) {
        val file = File(path)
        if (file.length() == 0L) {
            return
        }

        if ("gif" == FileUtils.getFileNameExtension(path)) {
            giv = GifImageViewExt(context)
            giv?.scaleType = ImageView.ScaleType.CENTER_INSIDE
            addView(giv, ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
            val layoutParams = giv?.layoutParams as LayoutParams
            layoutParams.gravity = Gravity.CENTER
            giv?.layoutParams = layoutParams
            giv?.initGif(File(path))
            giv?.start()
        } else {
            val iv = ImageView(context)
            addView(iv, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
            // 有需要再加
            ImageLoader.getInstance().displayImage("file://$path", iv, object : ImageLoadingListener {
                override fun onLoadingComplete(p0: String?, p1: View?, p2: Bitmap?) {
                    val attacher = PhotoViewAttacher(iv)
                    attacher.update()
                }

                override fun onLoadingStarted(p0: String?, p1: View?) {}

                override fun onLoadingCancelled(p0: String?, p1: View?) {}

                override fun onLoadingFailed(p0: String?, p1: View?, p2: FailReason?) {}
            })
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun onDestroy() {
        giv?.destroy()
    }

    companion object {

        private const val TAG = "ImagePreviewView"

    }
}